@@ -2,14 +2,14 @@
     MultiParamTypeClasses, GeneralizedNewtypeDeriving #-}
 module Test.Tasty.Golden.Internal where
 
-import Data.Typeable (Typeable)
 import Control.Applicative
 import Control.Monad.Cont
-import Test.Tasty.Providers
-import Data.ByteString.Lazy as LB
+import Control.DeepSeq
 import Control.Exception
+import Data.Typeable (Typeable)
+import Data.ByteString.Lazy as LB
 import System.IO
-import Data.Maybe
+import Test.Tasty.Providers
 
 -- | See 'goldenTest' for explanation of the fields
 data Golden =
@@ -46,19 +46,21 @@ vgRun :: ValueGetter r r -> IO r
 vgRun (ValueGetter a) = runContT a evaluate
 
 instance IsTest Golden where
-  run opts golden _ = runGolden golden
+  run _ golden _ = runGolden golden
   testOptions = return []
 
 runGolden :: Golden -> IO Result
 runGolden (Golden getGolden getTested cmp _) = do
-  result <- vgRun $ do
+  vgRun $ do
     new <- getTested
     ref <- getGolden
-    liftIO $ cmp ref new
+    result <- liftIO $ cmp ref new
 
-  return $
     case result of
-      Just reason ->
-        Result False reason
+      Just reason -> do
+        -- Make sure that the result is fully evaluated and doesn't depend
+        -- on yet un-read lazy input
+        liftIO $ evaluate . rnf $ reason
+        return $ testFailed reason
       Nothing ->
-        Result True ""
+        return $ testPassed ""
